import os
import re
import subprocess
from datetime import datetime
import xml.etree.ElementTree as ET

# Current date and time
current_datetime = datetime.now().strftime("%Y%m%d_%H%M%S")


def create_vdenoiser_xml(file_name, settings):
    # Create the root element
    root = ET.Element("vdenoiser")

    # Loop through the dictionary and create elements for each setting
    for key, value in settings.items():
        element = ET.SubElement(root, key)
        element.text = str(value)

    # Create an ElementTree object from the root element
    tree = ET.ElementTree(root)

    # Write the tree to an XML file
    tree.write(file_name)


def format_sequence_path(file_path):
    # Check if the path is a specific file and convert it to sequence format
    if file_path.endswith(".exr") and re.search(r'\.\d{4}\.exr$', file_path):
        return re.sub(r'\d{4}(\.exr)$', '????\\1', file_path)
    return file_path


def get_file_paths():
    vdenoiser_path = input(r"Enter the full path to the vdenoiser tool executable (e.g., 'C:/path/to/vdenoiser.exe'): ")
    # r"C:\Program Files\Maxon Cinema 4D 2023\plugins\V-Ray\tools\vdenoise.exe"

    input_path = input(
        "Enter the path for the input file or directory (e.g., 'path/to/your/file.exr' or 'path/to/your/directory/'): ")

    output_path = input("Enter the path for the output file (leave blank to use the same pattern as input): ")

    # Determine if input is a directory or specific file and format accordingly
    if os.path.isdir(input_path):
        input_file = os.path.join(input_path, "????").replace("\\", "/") + ".exr"
    else:
        input_file = format_sequence_path(input_path)

    # Apply similar logic to output file, defaulting to input file's pattern if not specified
    if output_path:
        if os.path.isdir(output_path):
            output_file = os.path.join(output_path, "????").replace("\\", "/") + ".exr"
        else:
            output_file = format_sequence_path(output_path)
    else:
        output_file = input_file  # Use the same format as input file if no output path is provided

    return vdenoiser_path, input_file, output_file


def select_mode():
    modes = ['strong', 'default', 'mild']
    mode = input("Choose the mode (strong, default, mild): ").strip().lower()
    while mode not in modes:
        print("Invalid mode selected.")
        mode = input("Choose the mode (strong, default, mild): ").strip().lower()
    return mode


def radius_boost():
    boost = input("Enter the radius boost value (0 .. 10 pixels): ").strip().lower()
    # while boost not in list(range(0, 20)):
    #     print("Invalid boost value.")
    #     boost = input("Enter the radius boost value (0, 1, 2..): ").strip().lower()
    return boost


def auto_radius_boost():
    boost = input(
        "Enter the auto radius value (0 - No adjustment. 1 - Automatically adjusts the denoising radius.): ").strip().lower()
    while boost not in ['0', '1']:
        print("Invalid boost value.")
        boost = input(
            "Enter the auto radius value (0 - No adjustment. 1 - Automatically adjusts the denoising radius.): ").strip().lower()
    return boost


def run_vdenoiser(settings):
    command = [settings['vdenoiserPath']]
    for key, value in settings.items():
        if key != 'vdenoiserPath':
            command.append(f'-{key}="{value}"')

    print("Running command:", ' '.join(command))

    # Execute the command
    try:
        subprocess.run(command, check=True)
        print("Vdenoiser completed successfully.")
    except subprocess.CalledProcessError as e:
        print("An error occurred while running vdenoiser:", str(e))


# Get file paths and vdenoiser tool path from user
vdenoiser_path, input_file, output_file = get_file_paths()

# Select the mode for the vdenoiser
selected_mode = select_mode()

# Select the radius boost for the vdenoiser
radius_boost = radius_boost()

# Select the radius boost for the vdenoiser
auto_radius_boost = auto_radius_boost()

# Define the settings for the command line
vdenoiser_settings = {"vdenoiserPath": vdenoiser_path,

    "inputFile": input_file,

    "outputFile": output_file,

    # "useGpu": "0",  # 0 | 1 | 2 0 - Use CPU. (default)

    "mode": selected_mode,  # strong | default | mild

    "boost": "2",  # 0 - No boost. (default) 1 - Moderate boost. 2 - Large boost.

    "skipExisting": "0", # 1 - Skip: The input image is skipped and the existing output image is not affected.

    "frames": "", # frames= 1-200, 2" - the denoiser works with every second frame from 1 to 200.

    "denoiseElements": "all", # For example, "rgb color;vrayreflection;*lightselect*" will denoise the RGB Color,

    "verboseLevel": "3",

    "outputDirectory": os.path.dirname(output_file),

    "strength": "1", # Larger values remove noise more aggressively but might blur the image too much.
    # The default is 1.0, which uses the noise in the noiseLevel Render Element

    "radius": radius_boost,
    # Specifies pixel radius for denoising. Larger values slow down the denoiser but might produce smoother results.
    # The default is 10 pixels, which corresponds to the Default preset.

    "frameBlend": "0",

    "oclquery": "default",

    "strips": "4",

    "autoRadius": auto_radius_boost,  # |0 - No adjustment. 1 - Automatically adjusts the denoising radius.

    "threshold": "0.5",

    "memLimit": "4096",

    "outputOffset": "0"

}

# Run the vdenoiser tool
run_vdenoiser(vdenoiser_settings)

# Create XML configuration
create_vdenoiser_xml("vdenoiser_config.xml", vdenoiser_settings)
